@@ -19,6 +19,7 @@ package simulator
 import (
 	"fmt"
 	"strings"
+	"sync"
 
 	apiv1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -44,28 +45,46 @@ const (
 	affinityPredicateName = "MatchInterPodAffinity"
 )
 
-type predicateInfo struct {
-	name      string
-	predicate predicates.FitPredicate
+var (
+	// initMutex is used for guarding static initialization.
+	staticInitMutex sync.Mutex
+	// statiInitHappened denotes if static initialization happened.
+	staticInitDone bool
+)
+
+// PredicateInfo assigns a name to a predicate
+type PredicateInfo struct {
+	Name      string
+	Predicate predicates.FitPredicate
 }
 
 // PredicateChecker checks whether all required predicates pass for given Pod and Node.
 type PredicateChecker struct {
-	predicates                []predicateInfo
+	predicates                []PredicateInfo
 	predicateMetadataProducer predicates.PredicateMetadataProducer
 	enableAffinityPredicate   bool
 }
 
+// We run some predicates first as they are cheap to check and they should be enough
+// to fail predicates in most of our simulations (especially binpacking).
 // There are no const arrays in Go, this is meant to be used as a const.
-var priorityPredicates = []string{"PodFitsResources", "GeneralPredicates", "PodToleratesNodeTaints"}
+var priorityPredicates = []string{"PodFitsResources", "PodToleratesNodeTaints", "GeneralPredicates", "ready"}
+
+func staticInitIfNeeded() {
+	staticInitMutex.Lock()
+	defer staticInitMutex.Unlock()
+
+	if staticInitDone {
+		return
+	}
 
-func init() {
 	// This results in filtering out some predicate functions registered by defaults.init() method.
 	// In scheduler this method is run from app.runCommand().
 	// We also need to call it in CA to have simulation behaviour consistent with scheduler.
 	// Note: the logic of method is conditional and depends on feature gates enabled. To have same
 	//       behaviour in CA and scheduler both need to be run with same set of feature gates.
 	algorithmprovider.ApplyFeatureGates()
+	staticInitDone = true
 }
 
 // NoOpEventRecorder is a noop implementation of EventRecorder
@@ -76,7 +95,7 @@ func (NoOpEventRecorder) Event(object runtime.Object, eventtype, reason, message
 }
 
 // Eventf is a noop method implementation
-func (NoOpEventRecorder) Eventf(object runtime.Object, eventtype, reason, messageFmt string, args ...interface{}) {
+func (NoOpEventRecorder) Eventf(regarding runtime.Object, related runtime.Object, eventtype, reason, action, note string, args ...interface{}) {
 }
 
 // PastEventf is a noop method implementation
@@ -89,6 +108,8 @@ func (NoOpEventRecorder) AnnotatedEventf(object runtime.Object, annotations map[
 
 // NewPredicateChecker builds PredicateChecker.
 func NewPredicateChecker(kubeClient kube_client.Interface, stop <-chan struct{}) (*PredicateChecker, error) {
+	staticInitIfNeeded()
+
 	informerFactory := informers.NewSharedInformerFactory(kubeClient, 0)
 	algorithmProvider := factory.DefaultProvider
 
@@ -103,8 +124,9 @@ func NewPredicateChecker(kubeClient kube_client.Interface, stop <-chan struct{})
 	serviceInformer := informerFactory.Core().V1().Services()
 	pdbInformer := informerFactory.Policy().V1beta1().PodDisruptionBudgets()
 	storageClassInformer := informerFactory.Storage().V1().StorageClasses()
+	csiNodeInformer := informerFactory.Storage().V1beta1().CSINodes()
+
 	configurator := factory.NewConfigFactory(&factory.ConfigFactoryArgs{
-		SchedulerName:                  apiv1.DefaultSchedulerName,
 		Client:                         kubeClient,
 		NodeInformer:                   nodeInformer,
 		PodInformer:                    podInformer,
@@ -135,37 +157,37 @@ func NewPredicateChecker(kubeClient kube_client.Interface, stop <-chan struct{})
 	sched := scheduler.NewFromConfig(config)
 
 	scheduler.AddAllEventHandlers(sched, apiv1.DefaultSchedulerName,
-		nodeInformer, podInformer, pvInformer, pvcInformer, replicationControllerInformer, replicaSetInformer, statefulSetInformer, serviceInformer, pdbInformer, storageClassInformer)
+		nodeInformer, podInformer, pvInformer, pvcInformer, serviceInformer, storageClassInformer, csiNodeInformer)
 
 	predicateMap := map[string]predicates.FitPredicate{}
-	for predicateName, predicateFunc := range sched.Config().Algorithm.Predicates() {
+	for predicateName, predicateFunc := range sched.Algorithm.Predicates() {
 		predicateMap[predicateName] = predicateFunc
 	}
-	predicateMap["ready"] = isNodeReadyAndSchedulablePredicate
-	if err != nil {
-		return nil, err
-	}
-	// We always want to have PodFitsResources as a first predicate we run
-	// as this is cheap to check and it should be enough to fail predicates
+	// We want to make sure that some predicates are present to run them first
+	// as they are cheap to check and they should be enough to fail predicates
 	// in most of our simulations (especially binpacking).
+	predicateMap["ready"] = IsNodeReadyAndSchedulablePredicate
 	if _, found := predicateMap["PodFitsResources"]; !found {
 		predicateMap["PodFitsResources"] = predicates.PodFitsResources
 	}
+	if _, found := predicateMap["PodToleratesNodeTaints"]; !found {
+		predicateMap["PodToleratesNodeTaints"] = predicates.PodToleratesNodeTaints
+	}
 
-	predicateList := make([]predicateInfo, 0)
+	predicateList := make([]PredicateInfo, 0, len(predicateMap))
 	for _, predicateName := range priorityPredicates {
 		if predicate, found := predicateMap[predicateName]; found {
-			predicateList = append(predicateList, predicateInfo{name: predicateName, predicate: predicate})
+			predicateList = append(predicateList, PredicateInfo{Name: predicateName, Predicate: predicate})
 			delete(predicateMap, predicateName)
 		}
 	}
 
 	for predicateName, predicate := range predicateMap {
-		predicateList = append(predicateList, predicateInfo{name: predicateName, predicate: predicate})
+		predicateList = append(predicateList, PredicateInfo{Name: predicateName, Predicate: predicate})
 	}
 
 	for _, predInfo := range predicateList {
-		klog.V(1).Infof("Using predicate %s", predInfo.name)
+		klog.V(1).Infof("Using predicate %s", predInfo.Name)
 	}
 
 	informerFactory.Start(stop)
@@ -182,7 +204,8 @@ func NewPredicateChecker(kubeClient kube_client.Interface, stop <-chan struct{})
 	}, nil
 }
 
-func isNodeReadyAndSchedulablePredicate(pod *apiv1.Pod, meta predicates.PredicateMetadata, nodeInfo *schedulernodeinfo.NodeInfo) (bool,
+// IsNodeReadyAndSchedulablePredicate checks if node is ready.
+func IsNodeReadyAndSchedulablePredicate(pod *apiv1.Pod, meta predicates.PredicateMetadata, nodeInfo *schedulernodeinfo.NodeInfo) (bool,
 	[]predicates.PredicateFailureReason, error) {
 	ready := kube_util.IsNodeReadyAndSchedulable(nodeInfo.Node())
 	if !ready {
@@ -194,9 +217,9 @@ func isNodeReadyAndSchedulablePredicate(pod *apiv1.Pod, meta predicates.Predicat
 // NewTestPredicateChecker builds test version of PredicateChecker.
 func NewTestPredicateChecker() *PredicateChecker {
 	return &PredicateChecker{
-		predicates: []predicateInfo{
-			{name: "default", predicate: predicates.GeneralPredicates},
-			{name: "ready", predicate: isNodeReadyAndSchedulablePredicate},
+		predicates: []PredicateInfo{
+			{Name: "default", Predicate: predicates.GeneralPredicates},
+			{Name: "ready", Predicate: IsNodeReadyAndSchedulablePredicate},
 		},
 		predicateMetadataProducer: func(_ *apiv1.Pod, _ map[string]*schedulernodeinfo.NodeInfo) predicates.PredicateMetadata {
 			return nil
@@ -204,6 +227,17 @@ func NewTestPredicateChecker() *PredicateChecker {
 	}
 }
 
+// NewCustomTestPredicateChecker builds test version of PredicateChecker with additional predicates.
+// Helps with benchmarking different ordering of predicates.
+func NewCustomTestPredicateChecker(predicateInfos []PredicateInfo) *PredicateChecker {
+	return &PredicateChecker{
+		predicates: predicateInfos,
+		predicateMetadataProducer: func(_ *apiv1.Pod, _ map[string]*schedulernodeinfo.NodeInfo) predicates.PredicateMetadata {
+			return nil
+		},
+	}
+}
+
 // SetAffinityPredicateEnabled can be used to enable or disable checking MatchInterPodAffinity
 // predicate. This will cause incorrect CA behavior if there is at least a single pod in
 // cluster using affinity/antiaffinity. However, checking affinity predicate is extremely
@@ -321,15 +355,15 @@ func (pe *PredicateError) PredicateName() string {
 func (p *PredicateChecker) CheckPredicates(pod *apiv1.Pod, predicateMetadata predicates.PredicateMetadata, nodeInfo *schedulernodeinfo.NodeInfo) *PredicateError {
 	for _, predInfo := range p.predicates {
 		// Skip affinity predicate if it has been disabled.
-		if !p.enableAffinityPredicate && predInfo.name == affinityPredicateName {
+		if !p.enableAffinityPredicate && predInfo.Name == affinityPredicateName {
 			continue
 		}
 
-		match, failureReasons, err := predInfo.predicate(pod, predicateMetadata, nodeInfo)
+		match, failureReasons, err := predInfo.Predicate(pod, predicateMetadata, nodeInfo)
 
 		if err != nil || !match {
 			return &PredicateError{
-				predicateName:  predInfo.name,
+				predicateName:  predInfo.Name,
 				failureReasons: failureReasons,
 				err:            err,
 			}
